@@ -5,7 +5,7 @@
  *
  * Copyright (C) 2005-2006  NTT DATA CORPORATION
  *
- * Version: 1.2   2006/09/03
+ * Version: 1.3.1   2007/01/05
  *
  * This file is applicable to both 2.4.30 and 2.6.11 and later.
  * See README.ccs for ChangeLog.
@@ -16,7 +16,9 @@
 #include <linux/ccs_common.h>
 #include <linux/sakura.h>
 #include <linux/realpath.h>
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,0)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,20)
+#include <linux/mount.h>
+#elif LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,0)
 #include <linux/namespace.h>
 #endif
 
@@ -38,11 +40,7 @@ static int AddNoUmountACL(const char *entry)
 {
 	NO_UMOUNT_ENTRY *new_entry, *ptr;
 	const char *cp;
-	if (!IsCorrectPath(entry, 1) || strendswith(entry, "/") == 0) {
-		printk(KERN_DEBUG "%s: Invalid pathname '%s'\n", __FUNCTION__, entry);
-		return -EINVAL;
-	}
-
+	if (!IsCorrectPath(entry, 1, 0, 1, __FUNCTION__)) return -EINVAL;
 	/* I don't want to add if it was already added. */
 	for (ptr = no_umount_list.next; ptr; ptr = ptr->next) if (strcmp(ptr->dir, entry) == 0) { ptr->is_deleted = 0; return 0; }
 	if ((cp = SaveName(entry)) == NULL || (new_entry = (NO_UMOUNT_ENTRY *) alloc_element(sizeof(NO_UMOUNT_ENTRY))) == NULL) goto out;
