@@ -5,7 +5,7 @@
  *
  * Copyright (C) 2005-2006  NTT DATA CORPORATION
  *
- * Version: 1.2   2006/09/03
+ * Version: 1.3.1   2006/12/08
  *
  * This file is applicable to both 2.4.30 and 2.6.11 and later.
  * See README.ccs for ChangeLog.
@@ -22,7 +22,7 @@
 #define ROOT_NAME "<kernel>"             /* A domain definition starts with <kernel> . */
 
 /* The initial domain. */
-struct domain_info KERNEL_DOMAIN = { NULL, NULL, ROOT_NAME, 0, 0 /* full_name_hash(ROOT_NAME) */ };
+struct domain_info KERNEL_DOMAIN = { NULL, NULL, ROOT_NAME, 0, 0, 0 /* full_name_hash(ROOT_NAME) */ };
 
 /* /sbin/init started? */
 extern int sbin_init_started;
@@ -60,21 +60,19 @@ typedef struct alias_entry {
 	const char *aliased_name;  /* Absolute pathname. Never NULL.                            */
 } ALIAS_ENTRY;
 
-/***** The structure for domain definitions to treat as trusted. *****/
+/***** The structure for domains to not to transit domains. *****/
 
-typedef struct trusted_pattern_entry {
-	struct trusted_pattern_entry *next; /* Pointer to next record. NULL if none.                       */
-	u32 hash_and_flag;                  /* (full_name_hash(trusted_pattern) << 1). LSB is delete flag. */
-	const char *trusted_pattern;        /* Pattern of domain name. Never NULL.                         */
-} TRUSTED_PATTERN_ENTRY;
+typedef struct domain_keeper_entry {
+	struct domain_keeper_entry *next; /* Pointer to next record. NULL if none.                     */
+	u32 hash_and_flag;                /* (full_name_hash(domain_keeper) << 1). LSB is delete flag. */
+	const char *domain_keeper;        /* Pattern of domain name. Never NULL.                       */
+} DOMAIN_KEEPER_ENTRY;
 
 /*************************  PROTOTYPES  *************************/
 
 static int IsInitializer(const char *filename);
-static int IsTrustedDomain(const char *domainname);
-static int IsCorrectDomain(const unsigned char *domainname);
+static int IsDomainKeeper(const char *domainname);
 static int AddInitializerEntry(const char *initializer);
-static void ReadPIDStatus(IO_BUFFER *head, const unsigned int attribute);
 
 /*************************  VARIABLES  *************************/
 
@@ -100,7 +98,7 @@ const char *GetLastName(const struct domain_info *domain)
  *  Check whether the given domainname follows the naming rules.
  *  Returns nonzero if follows, zero otherwise.
  */
-static int IsCorrectDomain(const unsigned char *domainname)
+int IsCorrectDomain(const unsigned char *domainname)
 {
 	unsigned char c, d, e;
 	if (!domainname || strncmp(domainname, ROOT_NAME, strlen(ROOT_NAME))) goto out;
@@ -143,7 +141,7 @@ void SetDomainAttribute(struct domain_info *domain, const u16 attribute)
 {
 	static spinlock_t lock = SPIN_LOCK_UNLOCKED;
 	spin_lock(&lock);
-	if (attribute == DOMAIN_ATTRIBUTE_UNTRUSTED) domain->attributes &= ~DOMAIN_ATTRIBUTE_TRUSTED;
+	if (attribute == DOMAIN_ATTRIBUTE_CLEAR_DOMAIN_KEEPER) domain->attributes &= ~DOMAIN_ATTRIBUTE_DOMAIN_KEEPER;
 	else domain->attributes |= attribute;
 	spin_unlock(&lock);
 }
@@ -167,6 +165,7 @@ int AddDomainACL(struct acl_info *ptr, struct domain_info *domain, struct acl_in
 	mb(); /* Instead of using spinlock. */
 	if (!ptr) domain->first_acl_ptr = (struct acl_info *) new_ptr;
 	else ptr->next = (struct acl_info *) new_ptr;
+	UpdateCounter(CCS_UPDATES_COUNTER_DOMAIN_POLICY);
 	return 0;
 }
 
@@ -174,6 +173,7 @@ int DelDomainACL(struct acl_info *prev, struct domain_info *domain, struct acl_i
 {
 	if (prev) prev->next = ptr->next;
 	else domain->first_acl_ptr = ptr->next;
+	UpdateCounter(CCS_UPDATES_COUNTER_DOMAIN_POLICY);
 	return 0;
 }
 
@@ -187,10 +187,7 @@ static int AddInitializerEntry(const char *initializer)
 	static spinlock_t lock = SPIN_LOCK_UNLOCKED;
 	const char *saved_initializer;
 	u32 hash_and_flag;
-	if (!IsCorrectPath(initializer, 0) || strendswith(initializer, "/")) {
-		printk(KERN_DEBUG "%s: Invalid pathname '%s'\n", __FUNCTION__, initializer);
-		return -EINVAL; /* No patterns allowed. */
-	}
+	if (!IsCorrectPath(initializer, 1, -1, -1, __FUNCTION__)) return -EINVAL; /* No patterns allowed. */
 	hash_and_flag = full_name_hash(initializer, strlen(initializer)) << 1;
 	/* I don't want to add if it was already added. */
 	for (ptr = initializer_list.next; ptr; ptr = ptr->next) if ((ptr->hash_and_flag & ~1) == hash_and_flag && strcmp(ptr->initializer, initializer) == 0) { ptr->hash_and_flag &= ~1; return 0; }
@@ -247,13 +244,7 @@ static int AddAliasEntry(const char *original_name, const char *aliased_name)
 	static spinlock_t lock = SPIN_LOCK_UNLOCKED;
 	const char *saved_original_name, *saved_aliased_name;
 	u32 hash_and_flag;
-	if (!IsCorrectPath(original_name, 0)) {
-		printk(KERN_DEBUG "%s: Invalid pathname '%s'\n", __FUNCTION__, original_name);
-		return -EINVAL;
-	} else if (!IsCorrectPath(aliased_name, 0)) {
-		printk(KERN_DEBUG "%s: Invalid pathname '%s'\n", __FUNCTION__, aliased_name);
-		return -EINVAL; /* No patterns allowed. */
-	}
+	if (!IsCorrectPath(original_name, 1, -1, -1, __FUNCTION__) || !IsCorrectPath(aliased_name, 1, -1, -1, __FUNCTION__)) return -EINVAL; /* No patterns allowed. */
 	hash_and_flag = full_name_hash(original_name, strlen(original_name)) << 1;
 	/* I don't want to add if it was already added. */
 	for (ptr = alias_list.next; ptr; ptr = ptr->next) if (strcmp(ptr->original_name, original_name) == 0 && strcmp(ptr->aliased_name, aliased_name) == 0) { ptr->hash_and_flag &= ~1; return 0; }
@@ -302,13 +293,7 @@ static int AddAggregatorEntry(const char *original_name, const char *aggregated_
 	static spinlock_t lock = SPIN_LOCK_UNLOCKED;
 	const char *saved_original_name, *saved_aggregated_name;
 	u32 depth_and_flag;
-	if (!IsCorrectPath(original_name, 1)) {
-		printk(KERN_DEBUG "%s: Invalid pathname '%s'\n", __FUNCTION__, original_name);
-		return -EINVAL;
-	} else if (!IsCorrectPath(aggregated_name, 0)) {
-		printk(KERN_DEBUG "%s: Invalid pathname '%s'\n", __FUNCTION__, aggregated_name);
-		return -EINVAL; /* No patterns allowed. */
-	}
+	if (!IsCorrectPath(original_name, 1, 0, -1, __FUNCTION__) || !IsCorrectPath(aggregated_name, 1, -1, -1, __FUNCTION__)) return -EINVAL;
 	depth_and_flag = PathDepth(original_name) << 1;
 	/* I don't want to add if it was already added. */
 	for (ptr = aggregator_list.next; ptr; ptr = ptr->next) if (strcmp(ptr->original_name, original_name) == 0 && strcmp(ptr->aggregated_name, aggregated_name) == 0) { ptr->depth_and_flag &= ~1; return 0; }
@@ -347,49 +332,45 @@ int AddAggregatorPolicy(char *data, const int is_delete)
 	return 0;
 }
 
-/*************************  TRUSTED DOMAIN HANDLER  *************************/
+/*************************  DOMAIN KEEPER HANDLER  *************************/
 
-static TRUSTED_PATTERN_ENTRY trusted_pattern_list = { NULL, 0, "" };
+static DOMAIN_KEEPER_ENTRY domain_keeper_list = { NULL, 0, "" };
 
-static int AddTrustedPatternEntry(const char *trusted_pattern)
+static int AddDomainKeeperEntry(const char *domain_keeper)
 {
-	TRUSTED_PATTERN_ENTRY *new_entry, *ptr;
+	DOMAIN_KEEPER_ENTRY *new_entry, *ptr;
 	u32 hash_and_flag;
-	if (!IsCorrectDomain(trusted_pattern)) {
-		printk(KERN_DEBUG "%s: Invalid domainname pattern '%s'\n", __FUNCTION__, trusted_pattern);
+	if (!IsCorrectDomain(domain_keeper)) {
+		printk(KERN_DEBUG "%s: Invalid domainname '%s'\n", __FUNCTION__, domain_keeper);
 		return -EINVAL;
 	}
-	hash_and_flag = full_name_hash(trusted_pattern, strlen(trusted_pattern)) << 1;
+	hash_and_flag = full_name_hash(domain_keeper, strlen(domain_keeper)) << 1;
 	/* I don't want to add if it was already added. */
-	for (ptr = trusted_pattern_list.next; ptr; ptr = ptr->next) {
-		if ((ptr->hash_and_flag & ~1) == hash_and_flag && strcmp(ptr->trusted_pattern, trusted_pattern) == 0) {
+	for (ptr = domain_keeper_list.next; ptr; ptr = ptr->next) {
+		if ((ptr->hash_and_flag & ~1) == hash_and_flag && strcmp(ptr->domain_keeper, domain_keeper) == 0) {
 			ptr->hash_and_flag &= ~1; break;
 		}
 	}
 	if (!ptr) {
 		static spinlock_t lock = SPIN_LOCK_UNLOCKED;
-		const char *saved_trusted_pattern;
-		if ((saved_trusted_pattern = SaveName(trusted_pattern)) == NULL || (new_entry = (TRUSTED_PATTERN_ENTRY *) alloc_element(sizeof(TRUSTED_PATTERN_ENTRY))) == NULL) return -ENOMEM;
+		const char *saved_domain_keeper;
+		if ((saved_domain_keeper = SaveName(domain_keeper)) == NULL || (new_entry = (DOMAIN_KEEPER_ENTRY *) alloc_element(sizeof(DOMAIN_KEEPER_ENTRY))) == NULL) return -ENOMEM;
 		new_entry->hash_and_flag = hash_and_flag;
-		new_entry->trusted_pattern = saved_trusted_pattern;
+		new_entry->domain_keeper = saved_domain_keeper;
 		/***** CRITICAL SECTION START *****/
 		spin_lock(&lock);
-		for (ptr = &trusted_pattern_list; ptr->next; ptr = ptr->next); ptr->next = new_entry;
+		for (ptr = &domain_keeper_list; ptr->next; ptr = ptr->next); ptr->next = new_entry;
 		spin_unlock(&lock);
 		/***** CRITICAL SECTION END *****/
 	}
 	/***** CRITICAL SECTION START *****/
-	{   /* Update trusted flag for existing domains. */
+	{   /* Update domain_keeper flag for existing domains. */
 		struct domain_info *domain;
-		const int len = strlen(trusted_pattern);
 		down(&new_domain_assign_lock);
 		for (domain = &KERNEL_DOMAIN; domain; domain = domain->next) {
-			if (GetDomainAttribute(domain) & DOMAIN_ATTRIBUTE_TRUSTED) continue;
-			if (strncmp(domain->domainname, trusted_pattern, len)) continue;
-			if (domain->domainname[len] == '\0' || domain->domainname[len] == ' ') {
-				SetDomainAttribute(domain, DOMAIN_ATTRIBUTE_TRUSTED);
-				printk("TOMOYO: Domain '%s' is now trusted.\n", domain->domainname);
-			}
+			if (GetDomainAttribute(domain) & DOMAIN_ATTRIBUTE_DOMAIN_KEEPER) continue;
+			if (strcmp(domain->domainname, domain_keeper)) continue;
+			SetDomainAttribute(domain, DOMAIN_ATTRIBUTE_DOMAIN_KEEPER);
 		}
 		up(&new_domain_assign_lock);
 	}
@@ -397,23 +378,20 @@ static int AddTrustedPatternEntry(const char *trusted_pattern)
 	return 0;
 }
 
-int AddTrustedPatternPolicy(char *trusted_pattern, const int is_delete)
+int AddDomainKeeperPolicy(char *domain_keeper, const int is_delete)
 {
-	TRUSTED_PATTERN_ENTRY *ptr;
-	if (!is_delete) return AddTrustedPatternEntry(trusted_pattern);
-	for (ptr = trusted_pattern_list.next; ptr; ptr = ptr->next) {
-		if (strcmp(ptr->trusted_pattern, trusted_pattern)) continue;
+	DOMAIN_KEEPER_ENTRY *ptr;
+	if (!is_delete) return AddDomainKeeperEntry(domain_keeper);
+	for (ptr = domain_keeper_list.next; ptr; ptr = ptr->next) {
+		if (strcmp(ptr->domain_keeper, domain_keeper)) continue;
 		ptr->hash_and_flag |= 1;
 		/***** CRITICAL SECTION START *****/
-		{   /* Update trusted flag for existing domains. */
+		{   /* Update domain_keeper flag for existing domains. */
 			struct domain_info *domain;
 			down(&new_domain_assign_lock);
 			for (domain = &KERNEL_DOMAIN; domain; domain = domain->next) {
-				if ((GetDomainAttribute(domain) & DOMAIN_ATTRIBUTE_TRUSTED) == 0) continue;
-				if (IsTrustedDomain(domain->domainname) == 0) {
-					SetDomainAttribute(domain, DOMAIN_ATTRIBUTE_UNTRUSTED);
-					printk("TOMOYO: Domain '%s' is now untrusted.\n", domain->domainname);
-				}
+				if ((GetDomainAttribute(domain) & DOMAIN_ATTRIBUTE_DOMAIN_KEEPER) == 0) continue;
+				if (IsDomainKeeper(domain->domainname) == 0) SetDomainAttribute(domain, DOMAIN_ATTRIBUTE_CLEAR_DOMAIN_KEEPER);
 			}
 			up(&new_domain_assign_lock);
 		}
@@ -422,46 +400,37 @@ int AddTrustedPatternPolicy(char *trusted_pattern, const int is_delete)
 	return 0;
 }
 
-int ReadTrustedPatternPolicy(IO_BUFFER *head)
+int ReadDomainKeeperPolicy(IO_BUFFER *head)
 {
-	TRUSTED_PATTERN_ENTRY *ptr = (TRUSTED_PATTERN_ENTRY *) head->read_var2;
-	if (!ptr) ptr = trusted_pattern_list.next;
+	DOMAIN_KEEPER_ENTRY *ptr = (DOMAIN_KEEPER_ENTRY *) head->read_var2;
+	if (!ptr) ptr = domain_keeper_list.next;
 	while (ptr) {
 		head->read_var2 = (void *) ptr;
-		if ((ptr->hash_and_flag & 1) == 0 && io_printf(head, KEYWORD_TRUST_DOMAIN "%s\n", ptr->trusted_pattern)) break;
+		if ((ptr->hash_and_flag & 1) == 0 && io_printf(head, KEYWORD_DOMAIN_KEEPER "%s\n", ptr->domain_keeper)) break;
 		ptr = ptr->next;
 	}
 	return ptr ? -ENOMEM : 0;
 }
 
-static int IsTrustedDomain(const char *domainname)
+static int IsDomainKeeper(const char *domainname)
 {
 	if (domainname) {
-		TRUSTED_PATTERN_ENTRY *ptr;
-		for (ptr = trusted_pattern_list.next; ptr; ptr = ptr->next) {
-			const int len = strlen(ptr->trusted_pattern);
-			if ((ptr->hash_and_flag & 1) || strncmp(domainname, ptr->trusted_pattern, len)) continue;
-			if (domainname[len] == ' ' || domainname[len] == '\0') return 1;
+		DOMAIN_KEEPER_ENTRY *ptr;
+		for (ptr = domain_keeper_list.next; ptr; ptr = ptr->next) {
+			if ((ptr->hash_and_flag & 1) || strcmp(domainname, ptr->domain_keeper)) continue;
+			return 1;
 		}
 	}
 	return 0;
 }
 
-int ReadTrustedPIDs(IO_BUFFER *head)
-{
-	if (!head->read_eof) {
-		ReadPIDStatus(head, DOMAIN_ATTRIBUTE_TRUSTED);
-		head->read_eof = 1;
-	}
-	return 0;
-}
-
 /*************************  DOMAIN DELETION HANDLER  *************************/
 
 static int SetDeleteMark(const char *domainname)
 {
 	struct domain_info *domain;
-	for (domain = &KERNEL_DOMAIN; domain; domain = domain->next) {
+	/* Never delete KERNEL_DOMAIN */
+	for (domain = KERNEL_DOMAIN.next; domain; domain = domain->next) {
 		if (GetDomainAttribute(domain) & DOMAIN_ATTRIBUTE_DELETED) continue;
 		if (strcmp(domain->domainname, domainname)) continue;
 		SetDomainAttribute(domain, DOMAIN_ATTRIBUTE_DELETED);
@@ -474,31 +443,6 @@ int DeleteDomain(char *data)
 	return SetDeleteMark(data);
 }
 
-int ReadDeletedPIDs(IO_BUFFER *head)
-{
-	if (!head->read_eof) {
-		ReadPIDStatus(head, DOMAIN_ATTRIBUTE_DELETED);
-		head->read_eof = 1;
-	}
-	return 0;
-}
-
-static void ReadPIDStatus(IO_BUFFER *head, const unsigned int attribute)
-{
-	struct task_struct *p;
-	/***** CRITICAL SECTION START *****/
-	read_lock(&tasklist_lock);
-	for_each_process(p) {
-		struct domain_info *domain = p->domain_info;
-		if (GetDomainAttribute(domain) & attribute) {
-			/* If there are too many process to list up, I can't sleep to reallocate, so I ignore them. */
-			if (io_printf(head, " %d", p->pid)) break;
-		}
-	}
-	read_unlock(&tasklist_lock);
-	/***** CRITICAL SECTION END *****/
-}
-
 /*************************  DOMAIN TRANSITION HANDLER  *************************/
 
 struct domain_info *FindDomain(const char *domainname)
@@ -516,7 +460,7 @@ struct domain_info *FindDomain(const char *domainname)
 	return NULL;
 }
 
-struct domain_info *FindOrAssignNewDomain(const char *domainname)
+struct domain_info *FindOrAssignNewDomain(const char *domainname, const u8 profile)
 {
 	struct domain_info *domain = NULL;
 	const char *saved_domainname;
@@ -527,11 +471,8 @@ struct domain_info *FindOrAssignNewDomain(const char *domainname)
 				struct domain_info *ptr = &KERNEL_DOMAIN;
 				domain->domainname = saved_domainname;
 				domain->hash = full_name_hash(saved_domainname, strlen(saved_domainname));
-				/* Check is_trusted flag for this domain. */
-				if (IsTrustedDomain(saved_domainname)) {
-					SetDomainAttribute(domain, DOMAIN_ATTRIBUTE_TRUSTED);
-					/* printk("TOMOYO: Created a trusted domain '%s'.\n", saved_domainname); */
-				}
+				domain->profile = profile;
+				if (IsDomainKeeper(saved_domainname)) SetDomainAttribute(domain, DOMAIN_ATTRIBUTE_DOMAIN_KEEPER);
 				mb(); /* Instead of using spinlock. */
 				while (ptr->next) ptr = ptr->next; ptr->next = domain;
 			}
@@ -572,7 +513,7 @@ static int Escape(char *dest, const char *src, int dest_len)
 	return -ENOMEM;
 }
 
-int FindNextDomain(const char *original_name, struct domain_info **next_domain, char __user * __user *argv)
+int FindNextDomain(const char *original_name, struct file *filp, struct domain_info **next_domain, char __user * __user *argv)
 {
 	/* This function assumes that the size of buffer returned by realpath() = PAGE_SIZE. */
 	struct domain_info *old_domain = current->domain_info, *domain = NULL;
@@ -650,7 +591,7 @@ int FindNextDomain(const char *original_name, struct domain_info **next_domain,
 	}
 	
 	/* Check execute permission. */
-	if ((retval = CheckFilePerm(real_program_name, 1, 1, "do_execve")) < 0) goto out;
+	if ((retval = CheckExecPerm(real_program_name, filp)) < 0) goto out;
 
 	/* Allocate memory for calcurating domain name. */
 	retval = -ENOMEM;
@@ -661,7 +602,7 @@ int FindNextDomain(const char *original_name, struct domain_info **next_domain,
 		snprintf(new_domain_name, PAGE_SIZE - 1, ROOT_NAME " " "%s", real_program_name);
 		if (strlen(new_domain_name) < PAGE_SIZE - 10) {
 			if (is_enforce) domain = FindDomain(new_domain_name);
-			else domain = FindOrAssignNewDomain(new_domain_name);
+			else domain = FindOrAssignNewDomain(new_domain_name, current->domain_info->profile);
 		}
 	} else if (old_domain == &KERNEL_DOMAIN && !sbin_init_started) {
 		/*
@@ -669,18 +610,18 @@ int FindNextDomain(const char *original_name, struct domain_info **next_domain,
 		 * But transit from kernel domain if executing initializers, for they might start before /sbin/init .
 		 */
 		domain = old_domain;
-	} else if ((GetDomainAttribute(old_domain) & DOMAIN_ATTRIBUTE_TRUSTED) == 0) {
+	} else if ((GetDomainAttribute(old_domain) & DOMAIN_ATTRIBUTE_DOMAIN_KEEPER) == 0) {
 		/* Normal domain transition. */
 		snprintf(new_domain_name, PAGE_SIZE - 1, "%s %s", old_domain->domainname, real_program_name);
 		if (strlen(new_domain_name) < PAGE_SIZE - 10) {
 			if (is_enforce) domain = FindDomain(new_domain_name);
-			else domain = FindOrAssignNewDomain(new_domain_name);
+			else domain = FindOrAssignNewDomain(new_domain_name, current->domain_info->profile);
 		}
 	} else {
 		domain = old_domain;
 	}
 	if (!domain && is_enforce && strlen(new_domain_name) < PAGE_SIZE - 10) {
-		if (CheckSupervisor("#Need to create domain\n%s\n", new_domain_name) == 0) domain = FindOrAssignNewDomain(new_domain_name);
+		if (CheckSupervisor("#Need to create domain\n%s\n", new_domain_name) == 0) domain = FindOrAssignNewDomain(new_domain_name, current->domain_info->profile);
 	}
 	if (!domain) {
 		if (is_enforce) retval = -EPERM;
