@@ -5,7 +5,7 @@
  *
  * Copyright (C) 2005-2006  NTT DATA CORPORATION
  *
- * Version: 1.3.1   2006/12/08
+ * Version: 1.3.1   2007/01/05
  *
  * This file is applicable to both 2.4.30 and 2.6.11 and later.
  * See README.ccs for ChangeLog.
@@ -29,7 +29,7 @@ static int AuditArgv0Log(const char *filename, const char *argv0, const int is_g
 	char *buf;
 	int len;
 	if (CanSaveAuditLog(is_granted) < 0) return -ENOMEM;
-	len = strlen(filename) + 8;
+	len = strlen(filename) + strlen(argv0) + 8;
 	if ((buf = InitAuditLog(&len)) == NULL) return -ENOMEM;
 	snprintf(buf + strlen(buf), len - strlen(buf) - 1, KEYWORD_ALLOW_ARGV0 "%s %s\n", filename, argv0);
 	return WriteAuditLog(buf, is_granted);
